@@ -2,15 +2,21 @@
 
 //! Machine learning, and dynamic automatic differentiation implementation.
 
-#[cfg(feature = "blas")]
-extern crate blas_src;
+extern crate mimalloc;
+
 #[cfg(feature = "blas")]
 extern crate cblas_sys;
+#[cfg(feature = "netlib")]
+extern crate netlib_src;
+#[cfg(feature = "openblas")]
+extern crate openblas_src;
 
 #[cfg(test)]
 #[macro_use]
 extern crate approx;
 
+use mimalloc::MiMalloc;
+
 pub mod numbers;
 #[macro_use]
 pub mod array;
@@ -23,6 +29,9 @@ pub mod layer;
 pub mod model;
 pub mod optimizer;
 
+#[global_allocator]
+static GLOBAL: MiMalloc = MiMalloc;
+
 #[cfg(test)]
 mod tests {
     use super::*;
